/**
 * Matrix
 *
 * This class creates a 2x2 matrix object, initialized with values
 * a  b
 * c  d
 * with the intent of displaying it and its transformations graphically.
 *
 * @author Sarah Greer
 * @see Plot
 *
 */


public class Matrix{
  Plot plot;        // plot object for display
  float a,b,c,d;    // matrix values
  float det;        // determinant (for calculation)
  float e1, e2;     // eigenvalues (for calculation)
  float[] one, two; // eigenvectors (for calculation)
  String determinant;   // string determinant (for display)
  String evalue1, evalue2;                  // string eigenvalues (for display)
  String evec1a, evec1b, evec2a, evec2b;    // string eigenvectors (for display)

  // variables used for plotting and calculations
  private Boolean isImaginary;
  private int nx;
  private float miny, minx, maxx, dy, dx;
  private float unitx, unity, sqt22x, sqt22y;
  private float cx, cy;
  private float[] map1, map2, map3, map4, map5, map6, map7, map8;

  public Matrix(Plot plot, float a, float b, float c, float d){
    this.plot = plot;
    graphUpdate();
    isImaginary = false;
    this.a = a;
    this.b = b;
    this.c = c;
    this.d = d;

    one = new float[2];
    two = new float[2];

    generateEval();
    l2Norm();

    plotGenerate();
  }

  public void setMatrix(float a, float b, float c, float d){
    this.a = a;
    this.b = b;
    this.c = c;
    this.d = d;
    generateEval();
    l2Norm();
    plotGenerate();
  }

  // normalize to the unit vector (for plotting)
  public void normalizeToOne(){
    float min1 = one[0];
    float min2 = two[0];
    if(one[1] < one[0]){ min1 = one[1]; } 
    if(two[1] < two[0]){ min2 = two[1]; } 

    one[0] /= min1;
    one[1] /= min1;

    two[0] /= min2;
    two[1] /= min2;
  }

  // L2 norm
  void l2Norm(){
    float onenorm = sqrt(one[0]*one[0] + one[1]*one[1]);
    float twonorm = sqrt(two[0]*two[0] + two[1]*two[1]);
    one[0] /= onenorm;
    one[1] /= onenorm;

    two[0] /= twonorm;
    two[1] /= twonorm;
  }

  // generate the eigenvalues and eigenvecotrs
  void generateEval(){
    float T = a + d;            // trace
    det = a*d - b*c;            // determinant
    determinant = str(det);
    float inRoot = T*T/4 - det; // value inside root
    float L1, L2;               // temporary storage values 

    // check for imaginary eigenvalues;
    if (inRoot >= 0){ 
      isImaginary = false;
      L1 = T/2 + sqrt(inRoot);
      L2 = T/2 - sqrt(inRoot);
      e1 = L1;
      e2 = L2;
      evalue1 = str(e1);
      evalue2 = str(e2);
    } else {
      isImaginary = true;
      L1 = 0;
      L2 = 0;
      e1 = L1;
      e2 = L2;
      evalue1 = str(T/2) + " + " + substr(str(sqrt(-1*inRoot)),8) + "i";
      evalue2 = str(T/2) + " - " + substr(str(sqrt(-1*inRoot)),8) + "i";
    }

    // calculate eigenvalues/eigenvectors.
    // doing this with a 2x2 matrix is easy...
    // case 1
    if (c != 0){
      one[0] = L1 - d;
      one[1] = c;
      two[0] = L2 - d;
      two[1] = c;
      if( ! isImaginary ){
        evec1a = substr(str(one[0]),8);
        evec1b = substr(str(one[1]),8);
        evec2a = substr(str(two[0]),8);
        evec2b = substr(str(two[1]),8);
      } else {
        float ft = T/2-d;
        if (ft != 0){
            evec1a = substr(str(ft),8) + " + " 
                     + substr(str(sqrt(-inRoot)),8) + "i";
            evec2a = substr(str(ft),8) + " - " 
                     + substr(str(sqrt(-inRoot)),8) + "i";
        } else{
            evec1a = substr(str(sqrt(-inRoot)),8) + "i";
            evec2a = substr(str(sqrt(-inRoot)),8) + "i";
        }
        evec1b = substr(str(c),8);
        evec2b = substr(str(c),8);
      }

    // case 2
    } else if (b != 0){
      one[0] = b;
      one[1] = L1 - a;
      two[0] = b;
      two[1] = L2 - a;
      if( ! isImaginary ){
        evec1a = substr(str(one[0]),8);
        evec1b = substr(str(one[1]),8);
        evec2a = substr(str(two[0]),8);
        evec2b = substr(str(two[1]),8);
      } else {
        float ft = T/2-a;
        if (ft != 0){
            evec1a = substr(str(ft),8) + " + " 
                     + substr(str(sqrt(-inRoot)),8) + "i";
            evec2a = substr(str(ft),8) + " - " 
                     + substr(str(sqrt(-inRoot)),8) + "i";
        } else{
            evec1a = substr(str(sqrt(-inRoot)),8) + "i";
            evec2a = substr(str(sqrt(-inRoot)),8) + "i";
        }
        evec1a = substr(str(b),8);
        evec2a = substr(str(b),8);
      }
    // case 3
    } else {
      one[0] = 1;
      one[1] = 0;
      two[0] = 0;
      two[1] = 1;
        evec1a = substr(str(one[0]),8);
        evec1b = substr(str(one[1]),8);
        evec2a = substr(str(two[0]),8);
        evec2b = substr(str(two[1]),8);
    }
  }
  
  // update the graph
  void graphUpdate(){
    nx = plot.nx;
    miny = plot.miny;
    minx = plot.minx;
    maxx = plot.maxx;
    dy = plot.dy;
    dx = plot.dx;
    unitx = 1/dx;
    unity = 1/dy;
    sqt22x = 1/dx * sqrt(2)/2;
    sqt22y = 1/dy * sqrt(2)/2;
    cx = plot.cx;
    cy = plot.cy;
    plotGenerate();

  }

  // generate the plot
  void plotGenerate(){
    // update
    map1 = transA(-unitx,0);
    map2 = transA(-sqt22x,sqt22y);
    map3 = transA(0,unity);
    map4 = transA(sqt22x,sqt22y);
    map5 = transA(unitx,0);
    map6 = transA(sqt22x,-sqt22y);
    map7 = transA(0,-unity);
    map8 = transA(-sqt22x,-sqt22y);
  }

  // apply the transformation on the given vector
  float[] transA(float x, float y){
    float x1 = a*x+b*y;
    float y1 = c*x+d*y;
    return new float[]{ x1, y1 };
  }

  // display the result
  void display(){
    // create ellipse
    // Yes, I do this by plotting 8 points and interpolating between them.
    // It's not perfect, but it works well enough
    // Alternatively, I could easily plot a square but it isn't as accurate of a
    // representation of the matrix
    noStroke();
    fill(255,255,0,210);
    beginShape();
        curveVertex(cx + map1[0], cy - map1[1]);
        curveVertex(cx + map1[0], cy - map1[1]);
        curveVertex(cx + map2[0], cy - map2[1]);
        curveVertex(cx + map3[0], cy - map3[1]);
        curveVertex(cx + map4[0], cy - map4[1]);
        curveVertex(cx + map5[0], cy - map5[1]);
        curveVertex(cx + map6[0], cy - map6[1]);
        curveVertex(cx + map7[0], cy - map7[1]);
        curveVertex(cx + map8[0], cy - map8[1]);
        curveVertex(cx + map1[0], cy - map1[1]);
        curveVertex(cx + map2[0], cy - map2[1]);
        curveVertex(cx + map3[0], cy - map3[1]);
        curveVertex(cx + map3[0], cy - map3[1]);
    endShape();

    // create eigenvectors
    strokeWeight(7.5);
    stroke(#00ff00);
    line(cx,cy, (cx + e1*one[0]*unitx), (cy - e1*one[1]*unity));
    stroke(#0000ff);
    line(cx,cy, (cx + e2*two[0]*unitx), (cy - e2*two[1]*unity));
    strokeWeight(1);
  }

}
