/**
 * This class creates a TextBox button that is used for number input
 *
 * @author Sarah Greer
 * @see Button
 *
 */


class TextBox extends Button{
  private boolean isSelected;
  private String txt;

  TextBox(float xpos, float ypos, float xsize, float ysize, String txt, color c){
    super(xpos, ypos, xsize, ysize, c);
    sw = 1;
    isSelected = false;
    this.txt = txt;
    txtsize = 15;
  }

  void checkSelected(){
    if ( isClicked() ){
      isSelected = true;
    } else if (mousePressed && !isHovering()){
      isSelected = false;
    }
  }

  void setNumber(String num){
    txt = num;
  }

  void display(){
    checkSelected();
    readKeys();

    stroke(c2);
    strokeWeight(sw);

    fill(c);
    rectMode(CENTER);
    rect(xpos,ypos,xsize,ysize);

    // selected textbox style
    if ( isSelected ){
      strokeWeight(3);
      stroke(#0080ff);
      line(xpos - xsize/2, ypos+ysize/2, xpos+xsize/2, ypos+ysize/2);

    // hovering textbox style
    } else if ( isHovering() ){
      fill(0,0,0,25);
      rectMode(CENTER);
      rect(xpos,ypos,xsize,ysize);
    }

    // add text to display
    if (txt != ""){
      textSize(txtsize);
      fill (c3);
      textAlign(CENTER, CENTER);
      text(txt, xpos, ypos);
    }
  }

  float parseToFloat(){
     return float(txt);
  }

  void readKeys(){
    if (keyPressed && isSelected) {
      // backspace functionality
      if (key== BACKSPACE) {
        if (txt.length() > 0) {
          txt = txt.substring(0, txt.length()-1);
        }
      }
      // add decimal
      else if (txt.indexOf('.') == -1 && key == '.' ){
        txt = txt + key;
      // delete all characters
      } else if (key== DELETE) {
        txt = "";
      // add negative sign
      } else if (txt.length() == 0 && key == '-'){
        txt = txt + key;
      // add number
      } else if ( key == '0' ||
                  key == '1' || 
                  key == '2' || 
                  key == '3' || 
                  key == '4' || 
                  key == '5' || 
                  key == '6' || 
                  key == '7' || 
                  key == '8' || 
                  key == '9' ) {
        txt = txt + key;
      }
    }
  }
}
