/**
 * 2x2 Matrix Visualization
 *
 * This is a program to visualize a 2x2 matrix and its equivalent two dimensional 
 * linear transformation. 
 *
 * It displays eigenvalues, eigenvectors, and the determinant.
 *
 * Made with Processing (https://processing.org/)
 *
 * @author Sarah Greer
 *
 * http://sygreer.com
 *
 */


// initialize plots and buttons
Plot plot;              // plot for matrix
Button r;               // reset button
Button submit;          // submit button

Matrix mat;             // matrix being displayed
TextBox A, B, C, D;     // text boxes of matrix

// selected
int selected_prev = 0;
int selected;

// zoom
int zoom_level = 0;
Button zin, zout;

private float xmin, xmax, ymin, ymax;   // current zoom value
private float xmin_orig, xmax_orig, ymin_orig, ymax_orig;   // default zoom values

void setup(){
  size(1200,800);
  background(255);

  // default zoom
  xmin = ymin = -2;
  xmax = ymax = 2;

  xmin_orig = xmin;
  xmax_orig = xmax;
  ymin_orig = ymin;
  ymax_orig = ymax;

  // initiaize plot, matrix, and buttons
  plot = new Plot(800,800,xmin, ymin, xmax, ymax, #d6ebf2);
  mat = new Matrix(plot, 1, 0, 0, 1);
  r = new Button(1000, 750, 100, 50, #ff0000);
  submit = new Button(1000, 225, 100, 50, #008000);
  A = new TextBox(950, 100, 70, 30, "1", #eeeeee);
  B = new TextBox(1050, 100, 70, 30, "0", #eeeeee);
  C = new TextBox(950, 150, 70, 30, "0", #eeeeee);
  D = new TextBox(1050, 150, 70, 30, "1", #eeeeee);

  zin = new Button(750, 50, 50, 50, #999999);
  zout = new Button(750, 100, 50, 50, #999999);

  r.setText(20, "Reset", #ffffff);
  submit.setText(20, "Submit", #ffffff);

  zin.setText(20, "+", #ffffff);
  zout.setText(20, "-", #ffffff);

  frameRate(10);
}

void draw(){
  background(#ffffff);
  if (selected_prev != selected){
    res();
  }

  selected_prev = selected;

  plot.display();
  mat.display();

  fill(#ffffff);
  noStroke();
  rect(1050,400,500,800);
  r.display();
  submit.display();
  A.display();
  B.display();
  C.display();
  D.display();

  // matrix container
  strokeWeight(3);
  stroke(#000000);
  line(905, 75, 905, 175); 
  line(905, 75, 915, 75); 
  line(905, 175, 915, 175); 
  line(1095, 75, 1095, 175); 
  line(1095, 75, 1085, 75); 
  line(1095, 175, 1085, 175); 

  zin.display();
  zout.display();

  if(r.isClicked()){
    res();
  }

  // display matrix information
  fill(#000000);
  textSize(20);
  textAlign(CENTER, CENTER);
  text("Eigenvalues", 910,350);
  text("Eigenvectors", 1090,350);
  line(820,365,1180,365);
  line(1000,340,1000,485);
  fill(#00ff00);
  text(mat.evalue1,910,400);
  text("[" + mat.evec1a + ", " + mat.evec1b + "]", 1090, 400);
  fill(#0000ff);
  text(mat.evalue2,910,450);
  text("[" + mat.evec2a + ", " + mat.evec2b + "]", 1090, 450);
  fill(#000000);
  text("Determinant: " + mat.determinant , 1000, 550);

  textSize(30);
  text("A  = ",860,125);
  textSize(15);
  textLeading(18);
  textAlign(LEFT,CENTER);

  if(submit.isClicked()){
    float a = A.parseToFloat();
    float b = B.parseToFloat();
    float c = C.parseToFloat();
    float d = D.parseToFloat();
    mat.setMatrix(a,b,c,d);
  }

  if(zin.isClicked() && zoom_level < 3){
    ymin += 0.5;
    ymax -= 0.5;
    xmin += 0.5;
    xmax -= 0.5;
    updateGraph();
    zoom_level +=1;
  }

  if(zout.isClicked()){ 
    ymin -= 0.5;
    ymax += 0.5;
    xmin -= 0.5;
    xmax += 0.5;
    updateGraph();
    zoom_level -=1;
  }

  textSize(20);
  textAlign(LEFT,CENTER);

}

// manipulate graph
void updateGraph(){
    plot.setXBounds(xmin,xmax);
    plot.setYBounds(ymin,ymax);
    mat.graphUpdate();
}

// reset the matrix and display
void res(){
    xmin = xmin_orig;
    xmax = xmax_orig;
    ymin = ymin_orig;
    ymax = ymax_orig;
    updateGraph();
    zoom_level = 0;
    
    mat.setMatrix(1,0,0,1);
    A.setNumber("1");
    B.setNumber("0");
    C.setNumber("0");
    D.setNumber("1");
}

// limit output string length
String substr(String string, int leng){
  if(string.length() > leng){
    string = string.substring(0,leng);
  }
  return string;
}
